/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.model;

import org.apache.commons.lang.StringUtils;
import org.hibernate.annotations.BatchSize;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import javax.persistence.*;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * JPA for maven project class
 */
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@Entity
@Table(name = "projects")
@NamedQueries(
    {
    @NamedQuery(name = MavenProject.SQL_SELECT_BY_MVN_ID, query = "SELECT p FROM MavenProject p WHERE p.groupId = :groupId AND p.artifactId = :artifactId AND p.branch = :branch AND p.enabled = true"),
    @NamedQuery(name = MavenProject.SQL_SELECT_BY_ID, query = "SELECT p FROM MavenProject p WHERE p.id = :id"),
    @NamedQuery(name = MavenProject.SQL_SELECT_BY_PARENT_PROJECT_ID, query = "SELECT p FROM MavenProject p WHERE p.parent.id = :id"),
    @NamedQuery(name = MavenProject.SQL_SELECT_DISABLED_PROJECTS, query = "SELECT p FROM MavenProject p WHERE p.enabled=false AND p.parent IS NULL")
    }
)
public class MavenProject implements Serializable {

  public final static String SQL_SELECT_BY_MVN_ID = "MavenProject.selectByMVNId";
  public final static String SQL_SELECT_BY_ID = "MavenProject.selectById";
  public static final String SQL_SELECT_BY_PARENT_PROJECT_ID = "MavenProject.selectByParentProjectId";
  public static final String SQL_SELECT_DISABLED_PROJECTS = "MavenProject.selectDisabledProjects";

  public final static String DEFAULT_BRANCH = "HEAD";

  private static final long serialVersionUID = -7747333646654989328L;
  public static final int DESCRIPTION_COLUMN_SIZE = 2000;
  public static final int NAME_COLUMN_SIZE = 256;

  @Id
  @Column(name = "id")
  @SequenceGenerator(name= "PROJECTS_SEQ", sequenceName = "PROJECTS_SEQ")
  @GeneratedValue(strategy = GenerationType.AUTO, generator = "PROJECTS_SEQ")
  private Integer id;

  @Column(name = "artifact_id", updatable = false, nullable = false, length = 256)
  private String artifactId;

  @Column(name = "group_id", updatable = false, nullable = false, length = 256)
  private String groupId;

  @Column(name = "name", updatable = true, nullable = true, length = NAME_COLUMN_SIZE)
  private String name;

  @Column(name = "description", updatable = true, nullable = true, length = DESCRIPTION_COLUMN_SIZE)
  private String description;

  @Column(name = "branch", updatable = false, nullable = false, length = 64)
  private String branch = DEFAULT_BRANCH;
  
  @Column(name = "enabled", updatable = true, nullable = false )
  private Boolean enabled;

  @ManyToOne(fetch = FetchType.LAZY, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH})
  @JoinColumn(name = "parent_project_id", updatable = true, nullable = true)
  @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
  private MavenProject parent;
  
  @OneToMany(mappedBy = "mavenProject", fetch = FetchType.LAZY, cascade = {CascadeType.ALL})
  @BatchSize(size = 8)
  private List<ProjectLink> projectLinks = new ArrayList<ProjectLink>();
  
  @OneToMany(mappedBy = "mavenProject", fetch = FetchType.LAZY, cascade = {CascadeType.ALL})
  @BatchSize(size = 32)
  private List<SnapshotGroup> snapshotGroups = new ArrayList<SnapshotGroup>();

  public MavenProject(String groupId, String artifactId, String name) {
    this();
    this.artifactId = artifactId;
    this.groupId = groupId;
    setName(name);
  }

  public MavenProject(String groupId, String artifactId, String name, MavenProject parent) {
    this(artifactId,groupId,name);
    this.parent = parent;
  }

  public MavenProject() {
    enabled = true;
  }
  
  public List<ProjectLink> getProjectLinks() {
    return projectLinks;
  }

  public void setProjectLinks(List<ProjectLink> projectLinks) {
    this.projectLinks = projectLinks;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = StringUtils.abbreviate(description, DESCRIPTION_COLUMN_SIZE);
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = StringUtils.abbreviate(name, NAME_COLUMN_SIZE);
  }

  public String getBranch() {
    return branch;
  }

  public void setBranch(String branch) throws IllegalArgumentException {
    checkNotNull(branch);
    this.branch = branch;
  }

  public MavenProject getParent() {
    return parent;
  }

  public void setParent(MavenProject parent) {
    this.parent = parent;
  }

  public String getArtifactId() {
    return artifactId;
  }

  public void setArtifactId(String artifactId) throws IllegalArgumentException {
    checkNotNull(artifactId);
    this.artifactId = artifactId;
  }

  public String getGroupId() {
    return groupId;
  }

  public void setGroupId(String groupId) throws IllegalArgumentException {
    checkNotNull(groupId);
    this.groupId = groupId;
  }

  public Integer getId() {
    return id;
  }

  public void setId(Integer id) {
    this.id = id;
  }

  public Boolean getEnabled() {
    return enabled;
  }

  public void setEnabled(Boolean enabled) {
    this.enabled = enabled;
  }

  public List<SnapshotGroup> getSnapshotGroups() {
    return snapshotGroups;
  }

  public void setSnapshotGroups(List<SnapshotGroup> snapshotGroups) {
    this.snapshotGroups = snapshotGroups;
  }

  private void checkNotNull(String attribute) {
    if (attribute == null || attribute.trim().length() == 0) {
      throw new IllegalArgumentException();
    }
  }
  
  public ProjectLink getProjectLinkByType( String type ) {
    if (type==null) {
      return null;
    }
    for (ProjectLink projectLink : projectLinks) {
      if ( type.equals(projectLink.getType())) {
        return projectLink;
      }
    }
    return null;
  }
}