/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.model;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.hibernate.annotations.BatchSize;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import javax.persistence.*;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Base class for a project metrics snapshot for a given maven plugin and
 * project
 */
@Entity
@Table(name = "snapshots")
@NamedQueries({@NamedQuery(name = Snapshot.SQL_SELECT_ALL, query = "SELECT m FROM Snapshot m, MavenProject pr"
    + " WHERE m.mavenProject = pr.id" + " AND pr.id = :idProject ORDER BY m.createdAt")
})
public class Snapshot implements Serializable {

  public final static String SQL_SELECT_ALL = "Snapshot.selectAll";

  private static final long serialVersionUID = -6637859483638004953L;

  @Id
  @Column(name = "id")
  @SequenceGenerator(name= "SNAPSHOTS_SEQ", sequenceName = "SNAPSHOTS_SEQ")
  @GeneratedValue(strategy = GenerationType.AUTO, generator = "SNAPSHOTS_SEQ")
  private Integer id;

  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "project_id", updatable = false, nullable = false)
  @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
  private MavenProject mavenProject;

  @ManyToOne(fetch = FetchType.LAZY)
  @JoinColumn(name = "snapshot_group_id", updatable = false, nullable = false)
  private SnapshotGroup snapshotGroup;

  @Temporal(TemporalType.TIMESTAMP)
  @Column(name = "created_at", updatable = false, nullable = false)
  private Date createdAt;

  @Column(name = "version", updatable = true, nullable = true, length = 32)
  private String version;

  @Column(name = "processed")
  private Boolean processed = Boolean.FALSE;
  
  @OneToMany(mappedBy = "snapshot", fetch = FetchType.LAZY, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REMOVE})
  @BatchSize(size = 128)
  private List<File> files = new ArrayList<File>(256);

  @OneToMany(mappedBy = "snapshot", fetch = FetchType.LAZY, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REMOVE})
  @BatchSize(size = 128)
  private List<ProjectMeasure> projectMeasures = new ArrayList<ProjectMeasure>(512);
  
  @OneToMany(mappedBy = "snapshot", fetch = FetchType.LAZY, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REMOVE})
  @BatchSize(size = 128)
  private List<ProjectTendency> tendencies = new ArrayList<ProjectTendency>(256);

  @OneToMany(mappedBy = "snapshot", fetch = FetchType.LAZY, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REMOVE})
  @BatchSize(size = 128)
  private List<FileMeasure> fileMeasures = new ArrayList<FileMeasure>(512);

  @OneToMany(mappedBy = "snapshot", fetch = FetchType.LAZY, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REMOVE})
  @BatchSize(size = 256)
  private List<RuleFailure> ruleFailures = new ArrayList<RuleFailure>(1024);
  
  @OneToMany(mappedBy = "snapshot", fetch = FetchType.LAZY, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REMOVE})
  private List<SnapshotLabel> labels = new ArrayList<SnapshotLabel>();
  
  public Snapshot() {
  }

  public Snapshot(MavenProject mavenProject, SnapshotGroup snapshotGroup, Date date) {
    this.mavenProject = mavenProject;
    this.snapshotGroup = snapshotGroup;
    this.createdAt = date;
  }

  public Boolean getProcessed() {
    return processed;
  }

  public void setProcessed(Boolean processed) {
    this.processed = processed;
  }

  public List<File> getFiles() {
    return files;
  }

  public void setFiles( List<File> files ) {
    this.files = files;
  }
  
  public List<FileMeasure> getFileMeasures() {
    return fileMeasures;
  }

  public void setFileMeasures(List<FileMeasure> fileMeasures) {
    this.fileMeasures = fileMeasures;
  }

  public List<ProjectMeasure> getProjectMeasures() {
    return projectMeasures;
  }

  public void setProjectMeasures(List<ProjectMeasure> projectMeasures) {
    this.projectMeasures = projectMeasures;
  }

  public List<RuleFailure> getRuleFailures() {
    return ruleFailures;
  }

  public List<RuleFailure> getRuleFailures(RuleFailureLevel level) {
    return getRuleFailures(level, null, null);
  }

  public void setRuleFailures(List<RuleFailure> ruleFailures) {
    this.ruleFailures = ruleFailures;
  }
  
  public Date getCreatedAt() {
    return createdAt;
  }

  public void setCreatedAt(Date createdAt) {
    this.createdAt = createdAt;
  }

  public Integer getId() {
    return id;
  }

  public void setId(Integer id) {
    this.id = id;
  }

  public MavenProject getMavenProject() {
    return mavenProject;
  }

  public void setMavenProject(MavenProject mavenProject) {
    this.mavenProject = mavenProject;
  }

  public String getVersion() {
    return version;
  }

  public void setVersion(String version) {
    this.version = version;
  }

  public SnapshotGroup getSnapshotGroup() {
    return snapshotGroup;
  }

  public void setSnapshotGroup(SnapshotGroup snapshotGroup) {
    this.snapshotGroup = snapshotGroup;
  }

  public List<ProjectTendency> getTendencies() {
    return tendencies;
  }

  public void setTendencies(List<ProjectTendency> tendencies) {
    this.tendencies = tendencies;
  }
  
  public List<SnapshotLabel> getLabels() {
    return labels;
  }

  public void setLabels(List<SnapshotLabel> labels) {
    this.labels = labels;
  }

  public List<RuleFailure> getRuleFailures(RuleFailureLevel level, Integer categId, Integer ruleId) {
    ArrayList<RuleFailure> result = new ArrayList<RuleFailure>();
    for (RuleFailure failure : getRuleFailures()) {
      if (failure.getLevel().equals(level)) {
        if (categId!=null && failure.getRule().getRulesCategory().getId().equals(categId)) {
          result.add(failure);
        } else if (ruleId!=null && failure.getRule().getId().equals(ruleId)) {
          result.add(failure);
        } else if (ruleId==null && categId==null) {
          result.add(failure);
        }
      }
    }
    return result;
  }

  public Map<File, Collection<FileMeasure>> getGroupedFileMeasures() {
    Map<File, Collection<FileMeasure>> files = new HashMap<File, Collection<FileMeasure>>();
    for (FileMeasure measure : getFileMeasures()) {
      File fle = measure.getFile();
      Collection<FileMeasure> measures = files.get(fle);
      if (measures == null) {
        measures = new ArrayList<FileMeasure>();
        files.put(fle, measures);
      }
      measures.add(measure);
    }
    return files;
  }

  public ProjectMeasure getProjectMeasure(Metric metric) {
    return getMetricMeasure(metric, getProjectMeasures(), null, null);
  }

  public List<ProjectMeasure> getProjectMeasuresOnCategories(Metric metric) {
    ArrayList<ProjectMeasure> result = new ArrayList<ProjectMeasure>();
    for (ProjectMeasure measure : getProjectMeasures()) {
      if (measure.getRuleId()==null && measure.getRulesCategoryId()!=null && measure.getMetric().equals(metric)) {
        result.add(measure);
      }
    }
    return result;
  }

  public List<ProjectMeasure> getProjectMeasuresOnRules(Metric metric) {
    ArrayList<ProjectMeasure> result = new ArrayList<ProjectMeasure>();
    for (ProjectMeasure measure : getProjectMeasures()) {
      if (measure.getRuleId()!=null && measure.getMetric().equals(metric)) {
        result.add(measure);
      }
    }
    return result;
  }

  public ProjectMeasure getProjectMeasure(Metric metric, Integer rulesCategoryId, Integer ruleId) {
    return getMetricMeasure(metric, getProjectMeasures(), rulesCategoryId, ruleId);
  }

  public FileMeasure getFileMeasure(Metric metric) {
    return getMetricMeasure(metric, getFileMeasures(), null, null);
  }

  public static <T extends MetricMeasure> T getMetricMeasure(Metric metric,
                                                             Collection<? extends MetricMeasure> measures,
                                                             Integer rulesCategoryId, Integer ruleId) {
    for (MetricMeasure measure : measures) {
      if (measure.getMetric().equals(metric)) {
        if (  (ruleId!=null && measure.getRuleId().equals(ruleId)) ||
              (ruleId==null && rulesCategoryId!=null && measure.getRulesCategoryId().equals(rulesCategoryId) && measure.getRuleId()==null) ||
              (ruleId==null && rulesCategoryId==null && measure.getRulesCategoryId()==null && measure.getRuleId()==null)) {
          return (T) measure;
        }
      }
    }
    return null;
  }

  public boolean equals(Object obj) {
    if (!(obj instanceof Snapshot)) {
      return false;
    }
    if (this == obj) {
      return true;
    }
    Snapshot other = (Snapshot) obj;
    return new EqualsBuilder()
        .append(id, other.getId()).isEquals();
  }

  public int hashCode() {
    return new HashCodeBuilder(17, 37).
        append(id).toHashCode();
  }
  
}