/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.service;

import org.apache.commons.lang.StringUtils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class WebInterfaceServiceImpl implements WebInterfaceService {

  private String remoteHostURL;
  
  protected final static int DOWNLOAD_FAILURE_RETRY_COUNT = 3;
  private static Map<String, byte[]> remotesFilesCache = Collections.synchronizedMap(new HashMap<String, byte[]>());
  private static final int CONNECT_TIMEOUT_MILLISECONDS = 30000;
  private static final int READ_TIMEOUT_MILLISECONDS = 60000;

  public WebInterfaceServiceImpl(String remoteHostURL) {
    this.remoteHostURL = remoteHostURL;
    if ( this.remoteHostURL.endsWith("/") ) {
      this.remoteHostURL = StringUtils.chop(this.remoteHostURL);
    }
  }
  
  public String getVersion() throws IOException {
    ByteArrayOutputStream target = new ByteArrayOutputStream();
    callRemoteFile("/remote_services/version", target, true);
    String version = new String(target.toByteArray());
    if ( version.trim().length() == 0 ) {
      throw new IOException("Empty version returned from server");
    }
    return version;
  }
  
  public boolean ping() {
    ByteArrayOutputStream target = new ByteArrayOutputStream();
    try {
      callRemoteFile("/remote_services/ping", target, false);
    } catch (IOException e) {
      return false;
    }
    return new String(target.toByteArray()).equals("ping");
  }
  
  public String getSonarMavenRepository() {
    return this.remoteHostURL + "/maven";
  }

  public boolean hasCheckstyleRulesExtensions() throws IOException {
    ByteArrayOutputStream target = new ByteArrayOutputStream();
    callRemoteFile("/remote_services/has_checkstyle_rules_extensions", target, false);
    return new String(target.toByteArray()).equals("true");
  }

  public boolean hasPmdRulesExtensions() throws IOException {
    ByteArrayOutputStream target = new ByteArrayOutputStream();
    callRemoteFile("/remote_services/has_pmd_rules_extensions", target, false);
    return new String(target.toByteArray()).equals("true");
  }

  public void getRemoteCheckstyleConfiguration(File target) throws IOException {
    callRemoteFile("/rules_configuration/export/checkstyle.xml", target, true);
  }

  public void getRemotePMDConfiguration(File target) throws IOException {
    callRemoteFile("/rules_configuration/export/pmd.xml", target, true);
  }

  public void triggerMeasuresCalculations() throws IOException {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    callRemoteFile("/batch_job/measures_calculations", out, false);
  }
  
  public InputStream getRemoteFile( URL url ) throws IOException {
    HttpURLConnection conn = (HttpURLConnection)url.openConnection();
    conn.setConnectTimeout(CONNECT_TIMEOUT_MILLISECONDS);
    conn.setReadTimeout(READ_TIMEOUT_MILLISECONDS);
    conn.connect();
    return conn.getInputStream();
  }
  
  public static void resetCache() {
    remotesFilesCache.clear();
  }
  
  private void callRemoteFile(String filePathOnServer, OutputStream target, boolean useCache) throws IOException {
    String fullFileURL = remoteHostURL + filePathOnServer;
    try {
      InputStream in = null;
      if ( useCache && remotesFilesCache.containsKey(filePathOnServer) ) {
        in = new ByteArrayInputStream( remotesFilesCache.get(filePathOnServer) );
      } else {
        int retryCount = 0;
        while ( in == null ) {
          try {
            in = getRemoteFile( new URL(fullFileURL) );
          } catch ( IOException ex ) {
            retryCount++;
            if ( retryCount == DOWNLOAD_FAILURE_RETRY_COUNT ) throw ex;
          }
        }
        if ( useCache ) {
          target = new CacheInterceptorOutputStream(target, remotesFilesCache, filePathOnServer);
        }
      }
      
      byte[] buffer = new byte[512];
      int readen;
      while ( ( readen = in.read( buffer ) ) != -1 ) {
        target.write( buffer, 0, readen );
      }
      in.close();
      target.close();
    } catch (IOException ex) {
      IOException ioEx = new IOException("Error occured during file " + fullFileURL + " download");
      ioEx.initCause(ex);
      throw ioEx;
    }
  }
  
  private void callRemoteFile(String filePathOnServer, File target, boolean useCache) throws IOException {
    FileOutputStream out = new FileOutputStream( target );
    callRemoteFile( filePathOnServer, out, useCache );
  }
  
  private class CacheInterceptorOutputStream extends OutputStream {

    private Map<String,byte[]> cache;
    
    private ByteArrayOutputStream cacheContent;
    private OutputStream intercepted;
    private String cacheKey;
    
    private CacheInterceptorOutputStream(OutputStream intercepted, Map<String,byte[]> cache, String cacheKey) {
      this.cache = cache;
      this.intercepted = intercepted;
      this.cacheKey = cacheKey;
      cacheContent = new ByteArrayOutputStream();
    }
    
    public void close() throws IOException {
      intercepted.close();
      cache.put(cacheKey, cacheContent.toByteArray() );
      cacheContent.close();
    }

    public void flush() throws IOException {
      intercepted.flush();
      cacheContent.flush();
    }

    public void write(byte[] b, int off, int len) throws IOException {
      intercepted.write(b, off, len);
      cacheContent.write(b, off, len);
    }

    public void write(byte[] b) throws IOException {
      intercepted.write(b);
      cacheContent.write(b);
    }

    public void write(int b) throws IOException {
      intercepted.write(b);
      cacheContent.write(b);
    }
  }

}
